
#include <PinChangeInterrupt.h>
#include <PinChangeInterruptBoards.h>
#include <PinChangeInterruptPins.h>
#include <PinChangeInterruptSettings.h>

//#include "Instrument_main.h"
#include "types.h"
#include "global_data.h"
#include "Menu.h"
#include "Interupts.h"
#include "Menu_types.h"
#include "Inputs.h"
#include "KeyMap.h"
#include "Logic.h"
#include "Outputs.h"
#include "Sodaq_wdt.h"

extern MENU_STATE_VARS  Menu_State;

//standard rate is 240 /m = 4Hz = means 4 x 6 values /sec = > 24 steps = 42ms 
// so if divisor is set to 3 => 130 / 3 = 43.3 ms -> 3.8Hz ?
// also need to be able to turn it on and off by button press as not legal at night.


// ===============================================================================
//  the whole messy PWM module has to be in the main.ino 

// going to use 2 pwm timers - 
// #1 for headlight modulation
// #2 for the PWM for the HEATED Grip  setting when used 

#if ( defined(__AVR_ATmega644__) || defined(__AVR_ATmega644A__) || defined(__AVR_ATmega644P__) || defined(__AVR_ATmega644PA__)  || \
        defined(ARDUINO_AVR_UNO) || defined(ARDUINO_AVR_NANO) || defined(ARDUINO_AVR_MINI) ||    defined(ARDUINO_AVR_ETHERNET) || \
        defined(ARDUINO_AVR_FIO) || defined(ARDUINO_AVR_BT)   || defined(ARDUINO_AVR_LILYPAD) || defined(ARDUINO_AVR_PRO)      || \
        defined(ARDUINO_AVR_NG) || defined(ARDUINO_AVR_UNO_WIFI_DEV_ED) || defined(ARDUINO_AVR_DUEMILANOVE) || defined(ARDUINO_AVR_FEATHER328P) || \
        defined(ARDUINO_AVR_METRO) || defined(ARDUINO_AVR_PROTRINKET5) || defined(ARDUINO_AVR_PROTRINKET3) || defined(ARDUINO_AVR_PROTRINKET5FTDI) || \
        defined(ARDUINO_AVR_PROTRINKET3FTDI) )
  #define USE_TIMER_1     true
#else          
  #define USE_TIMER_3     true
#endif

// These define's must be placed at the beginning before #include "AVR_Slow_PWM.h"
// _PWM_LOGLEVEL_ from 0 to 4
// Don't define _PWM_LOGLEVEL_ > 0. Only for special ISR debugging only. Can hang the system.
#define _PWM_LOGLEVEL_      0

#if (_PWM_LOGLEVEL_ > 3)
  #if USE_TIMER_1
    #warning Using Timer1
  #elif USE_TIMER_1
    #warning Using Timer3
  #endif
#endif

#define USING_MICROS_RESOLUTION       true    //false

// Default is true, uncomment to false
//#define CHANGING_PWM_END_OF_CYCLE     false

// // To be included only in main(), .ino with setup() to avoid `Multiple Definitions` Linker Error
#include "AVR_Slow_PWM.h"

// Don't change these numbers to make higher Timer freq. System can hang
#define HW_TIMER_INTERVAL_MS        0.1f
#define HW_TIMER_INTERVAL_FREQ      10000L

volatile uint32_t startMicros = 0;

// Init AVR_Slow_PWM, each can service 16 different ISR-based PWM channels
AVR_Slow_PWM ISR_PWM;  // the object

//////////////////////////////////////////////////////

#define USING_PWM_FREQUENCY     false //true

//////////////////////////////////////////////////////


// You can assign pins here. Be carefull to select good pin to use or crash
#define OUTPIN_CH1_HEAD  OUTPUT_PIN_LOW_BEAM
#define OUTPIN_CH2_HEAT  OUTPUT_PIN_SPARE

// You can assign any interval for any timer here, in microseconds
uint32_t HEAD_Period = 10000; // 100 Hz
uint32_t HEAT_Period = 20000;  // 50 Hz
// You can assign any duty_cycle for any PWM here, from 0-100
float HEAD_Duty  = 80.0f;     // 80 %
float HEAT_Duty  = 30.00f;    //90.0f;

// Channel number used to identify associated channel
int HEAD_Num;
int HEAT_Num;

uint8_t  HEAT_STEP_SIZE = 0;
bool wrote_flag = false;



void TimerHandler()
{ 
  ISR_PWM.run();
}

// ====================================================================================================


void Setup_Modulations(void)
{
 //  ISR_PWM.setPWM(PWM_Pin, PWM_Freq, PWM_DutyCycle, irqCallbackStartFunc, irqCallbackStopFunc);
  Serial.print(F("\nStarting ISR_Modify_PWM on ")); Serial.println(BOARD_NAME);
  Serial.println(AVR_SLOW_PWM_VERSION);
  Serial.print(F("CPU Frequency = ")); Serial.print(F_CPU / 1000000); Serial.println(F(" MHz"));

  ITimer1.init();

  // Using ATmega328 used in UNO => 16MHz CPU clock ,

  if (ITimer1.attachInterrupt(HW_TIMER_INTERVAL_FREQ, TimerHandler))
  {
    Serial.print(F("Starting  ITimer1 OK, micros() = ")); Serial.println(micros());
  }
  else
    Serial.println(F("Can't set ITimer1. Select another freq. or timer"));
    

   // configure pins to PWM outputs if needed
   if (TheConfigEpromData.Values[CONFIG_PARAM_MOD_LOW_BEAM].Bool == true)
     HEAD_Num = ISR_PWM.setPWM_Period(OUTPIN_CH1_HEAD, HEAD_Period, HEAD_Duty);  // microseconds 

   if (TheConfigEpromData.Values[CONFIG_PARAM_HEATED_GRIP_ON].Bool == true)
   {
    if (TheConfigEpromData.Values[CONFIG_PARAM_HEATED_GRIP_ON].Bool == true)
      HEAT_Num = ISR_PWM.setPWM_Period(OUTPIN_CH2_HEAT, HEAT_Period, HEAT_Duty);  // microseconds
   }
   else
    pinMode ( OUTPUT_PIN_SPARE, OUTPUT);   

 //  MOD_UPDATE_INTERVAL = (TheConfigEpromData.Values[CONFIG_PARAM_MOD_LOW_DEPTH].Num > 0) ? BASE_MOD_RATE / TheConfigEpromData.Values[CONFIG_PARAM_MOD_LOW_DEPTH].Num : BASE_MOD_RATE ;
   HEAT_STEP_SIZE = (TheConfigEpromData.Values[CONFIG_PARAM_HEATED_GRIP_STEPS].Num > 0) ? 100 / TheConfigEpromData.Values[CONFIG_PARAM_HEATED_GRIP_STEPS].Num : 33 ;   // default to 3 steps

  if (true == wrote_flag)
     Serial.println(F("WROTE_FLASH_DEFAULTS"));        
}





void setup() 
{
   uint32_t temp1,temp2;  // for compares
  sodaq_wdt_enable(WDT_PERIOD_2X);  // start up the WDT at 2s timeout
  Serial.begin(115200);  
  
  temp1 = Read_Var_From_Flash(STORED_SETUP_FLAG_ADDRESS);
#ifndef  FORCE_FLASH_WRITE
  if (temp1 != STORED_SETUP_FLAG_VALUE )
#endif
  { 
	  Copy_Data_To_Flash(TheConfigEpromData.Values[0].Num);
    Write_Byte_To_Flash(0, STORED_HEAT_SETTING_ADDRESS);   // store default heat setting   
    Write_Var_To_Flash(STORED_SETUP_FLAG_VALUE, STORED_SETUP_FLAG_ADDRESS);   
    wrote_flag = true;
  }  
  Copy_Data_From_Flash(0); // get from flash into the live region    


  Setup_Input_Pins();
  Setup_Output_Pins();

  // ###################
  //analogReadResolution(10);  // could set this to 1 bit /
  Setup_Modulations();

  Init_Menu(); // start the menu system  
  delay(100);
  
}
  



// small fns -==================

void Set_LOW_BEAM(uint8_t Sett)
{ // wrpapper to do on off or modulation change
  if (TheConfigEpromData.Values[CONFIG_PARAM_MOD_LOW_BEAM].Bool == true)
  { // if we are using the modulation function
     
     if (Sett == 0)
       HEAD_Duty = 0.0f;
     else if (Sett == 1)
       HEAD_Duty = 100.0f;
     else
      HEAD_Duty = (float) Sett;

      if (!ISR_PWM.modifyPWMChannel_Period (HEAD_Num, OUTPIN_CH1_HEAD, HEAD_Period, HEAD_Duty))
        Serial.print(F("modifyPWMChannel error for PWM_Period"));
  } 
  else // normal on off operation
  {
   if (Sett == false)
     digitalWrite ( OUTPUT_PIN_LOW_BEAM, OUTPUT_OFF);
   else
     digitalWrite ( OUTPUT_PIN_LOW_BEAM, OUTPUT_ON);
  }
}

void Set_SPARE_HEAT(uint8_t Sett)
{

  float fval; 

  if (TheConfigEpromData.Values[CONFIG_PARAM_HEATED_GRIP_ON].Bool == true)
  { // if we are using the GRIP functionfunction
    // calculate the actuall value that needs to be set for that percent of heat

    if (Sett == 0)
       HEAT_Duty = 0.0f;
    else if (Sett == 1)
       HEAT_Duty = 100.0f;
    else     
    {          
      HEAT_Duty = (float) Sett; 
      fval = (float) (Sett % 101) / 100;  // birn 0-1.0
      fval = sqrt(fval);          
      HEAT_Duty = 100 * fval; // scal back up to 0 -100 %
    }
           
    if (!ISR_PWM.modifyPWMChannel_Period (HEAT_Num, OUTPIN_CH2_HEAT, HEAT_Period, HEAT_Duty))
        Serial.print(F("modifyPWMChannel error for PWM_Period"));
  } 
  else // normal on off operation
  {    
    Serial.print(F("SEtting Spare Ooutput normal mode => "));
   if (Sett == false)
   {
     digitalWrite ( OUTPUT_PIN_SPARE, OUTPUT_OFF);
     Serial.println(F("LOW "));
   }
   else
   {
     digitalWrite ( OUTPUT_PIN_SPARE, OUTPUT_ON);
     Serial.println(F("HIGH"));
   }
  }
}

// then state counterseg brake flash




void loop() 
{
  bool refresh = false;
  bool keystroke = false;  
  uint32_t main_loop_time = 0;
  
  main_loop_time = millis();
  sodaq_wdt_reset();  // reset each time around the loop - should never be more than 0.5s
  
  // initialise and start the stepper motor driving system
  Process_Initial_State(); // set up events based on the  input states -
  Clear_Start_Interrupts();
   
  Do_Logic(); // work out the ouptuts based on input changes and time
  Do_Outputs(); // set the outputs and do autonomous timed functions (flash etc)
  DumpLogicStates();  
  
  while(1)
  {
    sodaq_wdt_reset();  // reset each time around the loop - should never be more than 0.5s
    
 #ifdef  STATE_TRACE_ON
      keystroke = Process_Keystroke(); 
           
      if (millis() - main_loop_time >= STATE_TRACE_INTERVAL)
      {
		  	Serial.write((unsigned char *)ANSI_CODE_Clear_Whole_Screen,4); // send the string 
        DumpLogicStates();                 
        main_loop_time = millis();
      }     
 #endif
 
 #ifdef MENU_CONFIG_ON
      keystroke = Process_Keystroke();
      if (keystroke == TRUE)
      {
        refresh = Start_Display_Refresh();
      }

      if ((refresh == TRUE) || (millis() - main_loop_time >= 250))
      {
        refresh = Update_Menu(refresh);
        main_loop_time = millis();
      }        
#endif    

    Do_Check_Inputs(); // process any input interrups and do poll   
    Do_Mapping();  // map 4 utton press if configured - otherwise doen nothing 
    Do_Logic(); // work out the ouptuts based on input changes and time
    Do_Outputs(); // set the outputs and do autonomous timed functions (flash etc)
    
  }
   // =================================================================================================== 
}





