//-------------------------------------------------------------------
#ifndef __KeyMap_H__
#define __KeyMap_H__
//-------------------------------------------------------------------
 
#include <arduino.h>

//-------------------------------------------------------------------
#define MODULATION_TOGGLE_MAX_DETECT_TIME 500     // low-high-low time < this toggles modulation if enabled.
#define MODULATION_TOGGLE_MIN_DETECT_TIME  100      // debounce

//bool Do_Normal_Logic(void);
//bool Do_4Button_Logic(void);

void  Do_Mapping(void);



// mapping of input button functions to output states is performed in the Logic module - 
// the output module takes care of autonomous timing operations and the input modules feed button/ swtich states 
// into the logic module





// for the 4 button mode - the functions are mapped - so there is a set of data defined for each button and the length of the press 

//-------------------------------------------------------------------

#define STARTER_ON_TIME  3000    // 3s
#define HORN_ON_TIME 2000        // 2s



#define NUM_TIMINGS 4

#define INDEX_SHORT  0
#define INDEX_DOUBLE 1
#define INDEX_MED    2
#define INDEX_LONG   3

#define NUM_BUTTONS 4


typedef enum
{
   BUTTON_UPPER_LEFT = 0,   // High
   BUTTON_LOWER_LEFT = 1 ,  // Horn
   BUTTON_UPPER_RIGHT = 2,  // Kill 
   BUTTON_LOWER_RIGHT = 3   // Start
} KEYMAP_BUTTON;





void Fn_High_Low_Tog(KEYMAP_BUTTON);
void Fn_Lights_Tog(KEYMAP_BUTTON);
void Fn_Modulation_Tog(KEYMAP_BUTTON);

void Fn_Heater_Up_step(KEYMAP_BUTTON);
void Fn_Heater_On_Tog(KEYMAP_BUTTON);

void Fn_Kill_ON_Toggle(KEYMAP_BUTTON);

void Fn_HORN_On_Time(KEYMAP_BUTTON);
void Fn_Starter_On_Time(KEYMAP_BUTTON);

void Fn_Left_Ind_Tog(KEYMAP_BUTTON);
void Fn_Left_Ind_Merge(KEYMAP_BUTTON);
void Fn_Right_Ind_Tog(KEYMAP_BUTTON);
void Fn_Right_Ind_Merge(KEYMAP_BUTTON);

//typedef void (*Funct_Ptr)(void);   // ptr to function void, taking void
typedef void (*Funct_Ptr)(KEYMAP_BUTTON);   // ptr to function void, taking void
 
/* -------------------- per menu ---------------------- */
 
// typedef Funct_Ptr MAP_ARRAY[NO_BUTTONS][NO_TIMINGS];

// typedef struct
// {
//   Funct_Ptr Times[NUM_TIMINGS];
// }  
// TIME_TYPES;


// typedef struct
// {
// 	 TIME_TYPES Button_Select[NUM_BUTTONS];
// } BUTTON_MAP_TYPE;

typedef struct
{
  Funct_Ptr  Execute[NUM_BUTTONS][NUM_TIMINGS];
  bool Timed_Horn;
  bool Timed_Starter;    
} BUTTON_MAP_TYPE;

// MAP 1
// Button,   | Short press < 1s, | Double press < 1s, | long Press 1 - 3s , | Extra long Press > 4s
// --------------------------//-----------------------------------------------
// Left Up   | High / low toggle,| Null,              | Modulation Toggle,  | Null
// Left Down | Left Ind Toggle,  | Fn_Left_Ind_Merge  | Horn                | Null 
// Right Up  | Heater Up         | Null               | Heater On /Off      | Lights toggle
// Right Down| Right Ind         | Fn_Left_Ind_Merge  | Toggle Starter      | KIll toggle
// timed_horn = true
// timed_Starter = true

// PROGMEM const BUTTON_MAP_TYPE Button_Map_1 = {{
// { Fn_High_Low_Tog, NULL, Fn_Modulation_Tog, NULL},
// { Fn_Left_Ind_Tog, NULL, Fn_HORN_On_Time,   NULL},
// { Fn_Heater_Up_step, NULL,  Fn_Heater_On_Tog, Fn_Lights_Tog},
// { Fn_Right_Ind_Tog, NULL, Fn_Starter_On_Time, Fn_Kill_ON_Toggle}},
// true,true,
// };  

PROGMEM const BUTTON_MAP_TYPE Button_Map_1 = {{
{ Fn_High_Low_Tog, Fn_HORN_On_Time, Fn_Modulation_Tog, NULL},
{ Fn_Left_Ind_Tog, Fn_Left_Ind_Merge ,Fn_Heater_On_Tog ,  NULL},
{ Fn_Heater_Up_step, Fn_Kill_ON_Toggle, Fn_Lights_Tog, Fn_Heater_On_Tog},
{ Fn_Right_Ind_Tog, Fn_Right_Ind_Merge, Fn_Starter_On_Time, NULL}},
false,false,
}; 

// MAP 2
// Button,   | Short press < 1s, | Double press < 1s, | long Press 1 - 3s , | Extra long Press > 4s
// --------------------------------------------------------------------------
// Left Up   | High / low toggle,| Null,              | Modulation Toggle,  | Null
// Left Down | Horn Toggle,      | Fn_Left_Ind_Merge  | Left Ind            | Null  
// Right Up  | Heater Up         | Null,              | Heater On /Off      | Lights toggle
// Right Down| KILL toggle       | Fn_RightInd_Merge  | Right Ind           | STARTER toggle
//-------------------------------------------------------------------
// timed_horn = false
// timed_Starter = true

PROGMEM const BUTTON_MAP_TYPE Button_Map_2 = {{
{ Fn_High_Low_Tog, NULL, Fn_Modulation_Tog, NULL},
{ Fn_HORN_On_Time, NULL, Fn_Left_Ind_Tog,   NULL },
{ Fn_Heater_Up_step, NULL, Fn_Heater_On_Tog, Fn_Lights_Tog},
{ Fn_Kill_ON_Toggle, NULL, Fn_Right_Ind_Tog, Fn_Starter_On_Time }},
false,true,
};  

  

// MAP 3
// Button,   | Short press < 1s, | Double press < 1s, | long Press 1 - 3s , | Extra long Press > 4s
// --------------------------------------------------------------------------
// Left Up   | Horn Toggle       | Null,              | High / low toggle   | Null
// Left Down | Left Ind          | Fn_Left_Ind_Merge  | Heater Up           | Heater On /Off    
// Right Up  | STARTER toggle    | Null,              | KILL toggle         | Null
// Right Down| Right Ind         | Fn_Right_Ind_Merge | Lights toggle       | Modulation Toggle 
//-------------------------------------------------------------------
// timed_horn = false
// timed_Starter = faslse

PROGMEM const BUTTON_MAP_TYPE Button_Map_3 = {{
{ Fn_HORN_On_Time, NULL, Fn_High_Low_Tog, NULL},
{ Fn_Left_Ind_Tog, NULL, Fn_Heater_Up_step, Fn_Heater_On_Tog },
{ Fn_Starter_On_Time, NULL, Fn_Kill_ON_Toggle, NULL},
{ Fn_Right_Ind_Tog, NULL, Fn_Lights_Tog, Fn_Modulation_Tog }},
false,false,
};  



// MAP 4
// Button,   | Short press < 1s, | Double press < 1s, | long Press 1 - 3s , | Extra long Press > 4s
// --------------------------------------------------------------------------
// Left Up   | High / low toggle,| Null,              | Modulation Toggle,  | Null
// Left Down | Horn Toggle,      | Fn_Left_Ind_Merge  | Left Ind            | Null  
// Right Up  | Heater Up         | Null,              | Heater On /Off      | Lights toggle
// Right Down| KILL toggle       | Fn_Right_Ind_Merge | Right Ind           | STARTER toggle
//-------------------------------------------------------------------
// timed_horn = false
// timed_Starter = true

PROGMEM const BUTTON_MAP_TYPE Button_Map_4 = {{
{ Fn_High_Low_Tog, NULL, Fn_Modulation_Tog, NULL},
{ Fn_HORN_On_Time, NULL, Fn_Left_Ind_Tog,   NULL },
{ Fn_Heater_Up_step, NULL, Fn_Heater_On_Tog, Fn_Lights_Tog},
{ Fn_Kill_ON_Toggle, NULL, Fn_Right_Ind_Tog, Fn_Starter_On_Time }},
false,true,
};  

#endif
//-------------------------------------------------------------------
 
 
