#include "arduino.h"
#include "types.h"
#include "global_data.h"
#include "Menu_types.h"
#include "Menu.h"
#include <EEPROM.h>

// each field is displayed in the string width

void Copy_Data_From_Flash(byte);
void Copy_Data_To_Flash(byte);
 
 
#define MENU_START_LINE   13 // line timing map display starts on

/* ---------------------------------------------------------- */
 
// -----------------------------------------
// we need to be able to refer to the stored data
// -----------------------------------------
 
#define DEBUG_ROW  5
#define DEBUG_COL  1
 
 
// static data -
MENU_STATE_VARS  Menu_State;
// this is the data that needs to be stored in the eeprom
DISPLAY_EEPROM_STORED_MENU_DATA  TheDisplayEpromData ;
CONFIG_EEPROM_STORED_MENU_DATA  TheConfigEpromData = {0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,
                                                            1,0,1,1,50,0,0,
                                                            0,1,20,3,1,5,0,1,5};
 
 
void Edit_Field_Value (const MENU_FIELD_ENTRY* fp, bool BigStep, bool Up, uint16_t maxval, uint16_t minval);
 
// ------------------------ Init Menu -------------
 
 
void Init_Menu()
{
	// return is used to indicate the finished state or not of the function
	Menu_State.Row_Index_Value = 0;
	Menu_State.Text_Index_Value = 0;
	Menu_State.Displayed = 0;
	Menu_State.Edit_Row = 0;
	Menu_State.Editing = 0;
	Menu_State.Current_Menu = ROOT_MENU;
	 
	// initialise data
	Copy_Data_From_Flash(TheConfigEpromData.Values[CONFIG_PARAM_PROFILE].Num);
	 
	// initialise output colour to black
	Serial.flush(); // empty the buffer
	Serial.write((unsigned char*)ANSI_CODE_Colour_Black,sizeof(ANSI_CODE_Colour_Black)-1);
	 
}

// --------------------------------------

//bool Check_Edit_Mode(byte row,byte column, uint16_t *val)
//{
//  
//    if((Menu_State.Current_Menu == INPUT_MENU) &&(Menu_State.Edit_Row == row))
//    {
//      *val =  TheDisplayEpromData.Values[row][column].Num;         //      
//      return (true);
//    }
//    return(false);   
//}
// 
// -------- utility fuctions -------------------

 
void Convert_Value_to_Str(int value, byte width, bool zeros)
{
	byte tenthous, thousands, hundreds, tens, units, outchar,counter;
	// rather than use printf we can use a simple routine to output the
	// ascii value
	// zeros is used to indicate we want leading zeros for decimals etc
	 
	tenthous = value / 10000;
	value = value - (tenthous * 10000);
	thousands = value / 1000;
	value = value - (thousands * 1000);
	hundreds = value / 100;
	value = value - (hundreds * 100);
	tens = value / 10;
	value = value - (tens * 10);
	units = value;
	// now to convert these to output in the string.
	// pad field if required

  for (counter = width; counter > 5; counter--)
	  Serial.write((unsigned char *)" ",1);

	if (width >= 5)
	{
		if ((tenthous == 0) && (zeros == FALSE))
		outchar = ' ';
		else
		{
			outchar = '0'+ tenthous;
			zeros = TRUE;   // after first written digit - zeros should be displayed
		}
		Serial.write((unsigned char *)&outchar,1);
		 
	}
	if (width >= 4)
	{
		if ((thousands == 0) && (zeros == FALSE))
		outchar = ' ';
		else
		{
			outchar = '0'+ thousands;
			zeros = TRUE;
		}
		Serial.write((unsigned char *)&outchar,1);
	}
	if (width >= 3)
	{
		if ((hundreds == 0) && (zeros == FALSE))
		outchar = ' ';
		else
		{
			outchar = '0'+ hundreds;
			zeros = TRUE;
		}
		Serial.write((unsigned char *)&outchar,1);
	}
	if (width >= 2)
	{
		if (((hundreds == 0) && (tens == 0)) && (zeros == FALSE))
		outchar = ' ';
		else
		{
			outchar = '0' + tens;
			zeros = TRUE;
		}
		Serial.write((unsigned char *)&outchar,1);
	}
	if (width >= 1)
	{
		outchar = '0' + units;
		Serial.write((unsigned char *)&outchar,1);
	}
}
 
 
void Display_Field (const MENU_FIELD_ENTRY *fp, byte x, byte y, byte Colr)
{
	// routine to take a particular field structure and display it
	// go to position
	// its going to display it into the output buffer at the position requested.
	// value to be displayed, x and y position, colour no 1-8,
	byte spaces;
	byte Display_Scale;
	byte frac_digits;
	int scaled_valint;
	int scaled_valfrac;
	VALUE_UNION_TYPE field;
	unsigned char bufer[MENU_FIELD_WIDTH];
	 
	// get the eprom value
	field = *((VALUE_UNION_TYPE *)(pgm_read_word_near(&(fp->Valptr)))); // ##### read from EEPROM
	 
	// postion the cursor - but if 0,0 leave where it is
	{
		Serial.write((unsigned char *)ANSI_CODE_Move_To_Position_begin,sizeof(ANSI_CODE_Move_To_Position_begin)-1);
		// insert y;x
		// these can possibly be mapped to as string predefines rather than calculated
		// we are avoiding printf at all costs.
	//	spaces = (y > 99) ? 2 : ;
    if (y> 99) 
      spaces = 3; 
    else if (y> 9) 
      spaces = 2;
    else
      spaces = 1;   
		Convert_Value_to_Str(y,spaces,FALSE);   // output the y location
		Serial.write((unsigned char *)";",1); // insert the separator
 //spaces = ((x)> 9) ? 2 : 1;     
	  if (x > 99) 
      spaces = 3; 
    else if (x > 9) 
      spaces = 2;
    else
      spaces = 1;
		Convert_Value_to_Str(x,spaces,FALSE);   // output the x location
	}
	Serial.write((unsigned char *)ANSI_CODE_Move_To_Position_end,sizeof(ANSI_CODE_Move_To_Position_end)-1);
	// set the colour

  //Colr = ((Menu_State.Text_Index_Value == (Menu_State.Edit_Row + CONFIG_MENU_OFFSET)) && (Menu_State.Editing == true )) ? MENU_EDIT_COLOUR : Colr;
  Colr = ((Menu_State.Text_Index_Value == Menu_State.Edit_Row) && (Menu_State.Editing == true )) ? MENU_EDIT_COLOUR : Colr; 

	switch (Colr)
	{
		case COLOUR_CODE_RED:
		Serial.write((unsigned char *)ANSI_CODE_Colour_Red,sizeof(ANSI_CODE_Colour_Red)-1);
		break;
		case COLOUR_CODE_GREEN:
		Serial.write((unsigned char *)ANSI_CODE_Colour_Green,sizeof(ANSI_CODE_Colour_Green)-1);
		break;
//		case COLOUR_CODE_YELLOW:
//		Serral.write((unsigned char *)ANSI_CODE_Colour_Yellow,sizeof(ANSI_CODE_Colour_Yellow)-1);
//		break;
		case COLOUR_CODE_BLUE:
		Serial.write((unsigned char *)ANSI_CODE_Colour_Blue,sizeof(ANSI_CODE_Colour_Blue)-1);
		break;
		case COLOUR_CODE_MAGENTA:
		Serial.write((unsigned char *)ANSI_CODE_Colour_Magenta,sizeof(ANSI_CODE_Colour_Magenta)-1);
		break;
//		case COLOUR_CODE_CYAN:
//		Serral.write((unsigned char *)ANSI_CODE_Colour_Cyan,sizeof(ANSI_CODE_Colour_Cyan)-1);
//		break;
//		case COLOUR_CODE_WHITE:
//		Serral.write((unsigned char *)ANSI_CODE_Colour_White,sizeof(ANSI_CODE_Colour_White)-1);
//		break;
//		case COLOUR_CODE_RESET:
//		Serral.write((unsigned char *)ANSI_CODE_Colour_Reset,sizeof(ANSI_CODE_Colour_Reset)-1);
//		break;
		case COLOUR_CODE_BLACK:
		default:
		Serial.write((unsigned char *)ANSI_CODE_Colour_Black,sizeof(ANSI_CODE_Colour_Black)-1);
		break;
	}
	// display the value
	// the code is done here to avoid another call which blows the stack
	switch (pgm_read_byte_near(&(fp -> var_type)))
	{
    case Display:
      // strncpy_P((char *)bufer,(char *)(DISPLAY_To_Str.DispSetting[field.Num]),DISPLAY_SEL_WIDTH);
       Serial.write((unsigned char *)bufer,DISPLAY_SEL_WIDTH);
    break;
		case Value:
		  Display_Scale = pgm_read_byte_near(&(fp->Display_Scale));
		  if (Display_Scale <= 1)
	  	{
	  		Convert_Value_to_Str(field.Num, MENU_FIELD_WIDTH,FALSE);
	  	}
	  	else
	  	{
			Display_Scale = pgm_read_byte_near(&(fp->Display_Scale));
			frac_digits = (Display_Scale > 10 ) ? 2 : 1;
			scaled_valint = field.Num / Display_Scale;
			scaled_valfrac = field.Num - (scaled_valint * Display_Scale);
			scaled_valfrac = (frac_digits == 2) ?
			(scaled_valfrac * 100) / Display_Scale :
			(scaled_valfrac * 10) / Display_Scale  ;  // decimal digit
			// firstly do the integer part
			Convert_Value_to_Str(scaled_valint,MENU_FIELD_WIDTH-(frac_digits+1),FALSE);
			// send decimal point
			Serial.write((unsigned char *)".",1); // send the string
			// then do a fractional part if needed
			Convert_Value_to_Str(scaled_valfrac,frac_digits,TRUE);
		}
		break;
		  case BOOL:
	  	// output the boolvalue as text
	  	if (field.Bool == TRUE)
      {
        strncpy_P((char *)bufer,(char *)PSTR("   TRUE\0"),8);
        Serial.write((unsigned char *)bufer,8);
      }   
  		else
      {
        strncpy_P((char *)bufer,(char *)PSTR("  FALSE\0"),8);
        Serial.write((unsigned char *)bufer,8);
      }   
		  break;
	}
 
	// revert the colour
	Serial.write((unsigned char *)ANSI_CODE_Colour_Black,sizeof(ANSI_CODE_Colour_Black)-1);
}
 
 
void Edit_Field_Value (const MENU_FIELD_ENTRY* fp, bool BigStep, bool Up, uint16_t maxval, uint16_t minval)
{
	uint16_t changeval;
	VALUE_UNION_TYPE changed;
	VALUE_UNION_TYPE origval;
	uint16_t MAXval,MINval,STEPval;
	 
	// get the value
	origval = *((VALUE_UNION_TYPE *)(pgm_read_word_near(&(fp->Valptr)))); // ##### read from EEPROM
	changed = origval;  // copy it
	 
	// calculate a new value and display it
	MAXval = pgm_read_word_near(&(fp-> Max_Val));
	MINval = pgm_read_word_near(&(fp-> Min_Val));
	STEPval = pgm_read_word_near(&(fp-> Step_Val));
	 
	changeval = (BigStep == TRUE)? 10 * STEPval: STEPval;
	 
	if (Up == TRUE)
	{
		changed.Num = (changed.Num <= (MAXval- changeval)) ? changed.Num + changeval : MAXval;
	}
	else
	{
		changed.Num = (changed.Num >= (MINval + changeval)) ? changed.Num - changeval : MINval;
	}
	 
	changed.Num = (changed.Num  >= maxval) ? maxval : changed.Num;
	changed.Num = (changed.Num <= minval) ? minval : changed.Num;
	 
	// update the value
	if (changed.Num != origval.Num)  // use num as largest in the union
	{  // write the updated value
		*((VALUE_UNION_TYPE *)(pgm_read_word_near(&(fp->Valptr)))) = changed;
	}
}
 
 
 
void  Display_Located_String (const LOCATED_STRING_TYPE * ptr)
{
	byte spaces;
	byte x,y;
  byte colour = MENU_EDIT_COLOUR;  
	unsigned char bufer[40];
	// move to postion  
	Serial.flush();
	{
		Serial.write((unsigned char *)ANSI_CODE_Move_To_Position_begin,sizeof(ANSI_CODE_Move_To_Position_begin)-1);
		// insert y;x
		// these can possibly be mapped to as string predefines rather than calculated
		// we are avoiding printf at all costs.
		x = pgm_read_byte_near(&(ptr->posx));
		y = pgm_read_byte_near(&(ptr->posy));
		spaces = (y>9) ? 2 : 1;
		Convert_Value_to_Str(y,spaces,FALSE);   // output the y location
		Serial.write((unsigned char* )";",1); // insert the separator
		spaces = (x>9) ? 2 : 1;
		Convert_Value_to_Str(x,spaces,FALSE);   // output the x location
	}
	Serial.write((unsigned char *)ANSI_CODE_Move_To_Position_end,sizeof(ANSI_CODE_Move_To_Position_end)-1);
	// set the colour
    
   // colour = ((Menu_State.Text_Index_Value == (Menu_State.Edit_Row + CONFIG_MENU_OFFSET)) && (Menu_State.Editing == true )) ? MENU_EDIT_COLOUR : pgm_read_byte_near(&(ptr->colour));
  colour = ((Menu_State.Text_Index_Value == Menu_State.Edit_Row)  && (Menu_State.Editing == true )) ? MENU_EDIT_COLOUR : pgm_read_byte_near(&(ptr->colour));

	// set colour
	switch (colour)
	{
		case COLOUR_CODE_RED:
		Serial.write((unsigned char *)ANSI_CODE_Colour_Red,sizeof(ANSI_CODE_Colour_Red)-1);
		break;
		case COLOUR_CODE_GREEN:
		Serial.write((unsigned char *)ANSI_CODE_Colour_Green,sizeof(ANSI_CODE_Colour_Green)-1);
		break;
//		case COLOUR_CODE_YELLOW:
//		Serral.write((unsigned char *)ANSI_CODE_Colour_Yellow,sizeof(ANSI_CODE_Colour_Yellow)-1);
//		break;
		case COLOUR_CODE_BLUE:
		Serial.write((unsigned char *)ANSI_CODE_Colour_Blue,sizeof(ANSI_CODE_Colour_Blue)-1);
		break;
		case COLOUR_CODE_MAGENTA:
		Serial.write((unsigned char *)ANSI_CODE_Colour_Magenta,sizeof(ANSI_CODE_Colour_Magenta)-1);
		break;
//		case COLOUR_CODE_CYAN:
//		Serral.write((unsigned char *)ANSI_CODE_Colour_Cyan,sizeof(ANSI_CODE_Colour_Cyan)-1);
//		break;
//		case COLOUR_CODE_WHITE:
//		Serral.write((unsigned char *)ANSI_CODE_Colour_White,sizeof(ANSI_CODE_Colour_White)-1);
//		break;
//		case COLOUR_CODE_RESET:
//		Serral.write((unsigned char *)ANSI_CODE_Colour_Reset,sizeof(ANSI_CODE_Colour_Reset)-1);
//		break;
		case COLOUR_CODE_BLACK:
		default:
		Serial.write((unsigned char *)ANSI_CODE_Colour_Black,sizeof(ANSI_CODE_Colour_Black)-1);
		break;
	}
	 
	strncpy_P((char *)bufer,(char *)ptr->text,CONFIG_STRING_WIDTH+1); // include terminator
	Serial.write((unsigned char *)bufer,CONFIG_STRING_WIDTH);
}
 
 
 
 
 
//###################################################################
 
// we need to generalise the display and edit functions -
// the display routine will simply output the whole lot -
// the edit function can just repaint the value being editted
 
void Reset_Menu_Vars(void)
{
	Menu_State.Edit_Row = 0;
	Menu_State.Editing = FALSE;
}
 
 
bool Start_Display_Refresh(void)
{
	Menu_State.Row_Index_Value = 0;  // start of line
	Menu_State.Text_Index_Value = 0;
	return(TRUE); // refresh ready
}
 
// ----------------- Display Menus ------------
// displays the menu as it passes around - one field per call
 
//static byte cursor_count = 0;
//static byte cursor_divide = 0;
 
bool Update_Menu(bool refreshing)
{
	byte COLOUR = COLOUR_CODE_BLACK;
	// really just should be a loop to display each item
	Serial.flush();
	if (refreshing == TRUE)   // ie sent the last line / segment
	{
	 
    if ((Menu_State.Text_Index_Value == Menu_State.Edit_Row) && (Menu_State.Editing != 0))
	//	if ((Menu_State.Text_Index_Value == (Menu_State.Edit_Row + CONFIG_MENU_OFFSET)) && (Menu_State.Editing != 0))
		{
			COLOUR = MENU_EDIT_COLOUR;
		}
     
		switch (Menu_State.Current_Menu)
		{
			case ROOT_MENU:
        Display_Located_String(&(Root_Menu.string_selections[Menu_State.Text_Index_Value]));
        if (++Menu_State.Text_Index_Value >= ROOT_MENU_LINES)
        {
          Menu_State.Text_Index_Value = 0;
          refreshing = FALSE;
        }
			break;
			 
			case CONFIG_SPECIAL_MENU:
        Display_Located_String(&(Config_Menu.string_selections[Menu_State.Text_Index_Value].Text_Stuff));
    
        // if theres a defined parameter then that needs to be displayed also
        if (pgm_read_word_near(&(Config_Menu.string_selections[Menu_State.Text_Index_Value].Value_Stuff.Valptr)) != NULL)
        {
          Serial.flush(); // make sure its been sent
          Display_Field (&(Config_Menu.string_selections[Menu_State.Text_Index_Value].Value_Stuff),
          55,
          pgm_read_byte_near(&(Config_Menu.string_selections[Menu_State.Text_Index_Value].Text_Stuff.posy)),
          pgm_read_byte_near(&(Config_Menu.string_selections[Menu_State.Text_Index_Value].Text_Stuff.colour)));
        }
        
        if (++Menu_State.Text_Index_Value >= CONFIG_MENU_LINES)
        {
          Menu_State.Text_Index_Value =  0;
          refreshing = FALSE;
        }
			break;
		}	 
	}
	else
	{
		Serial.write((unsigned char *)ANSI_CODE_Move_To_Position_begin,sizeof(ANSI_CODE_Move_To_Position_begin)-1);
		Serial.write((unsigned char *)"1;1",3); // insert the separator
		Serial.write((unsigned char *)ANSI_CODE_Move_To_Position_end,sizeof(ANSI_CODE_Move_To_Position_end)-1);
	// 	switch (cursor_count)
	// 	{
	// 		case 0: curs = '_'; break;
	// 		case 1: curs = 'o'; break;
	// 		case 2: curs = 'O'; break;
	// 		case 3: curs = 'o'; break;
	// 		default :curs= '!'; break;
	// 	}
	// 	cursor_count = (++cursor_count >= 4) ? 0 : cursor_count;
	// 	Serial.write((unsigned char *)&curs,1); // insert the separator
	// 	cursor_divide = 0;

	}
	return(refreshing);  // return if still refreshing
}
 
 

 
bool Process_RootMenu_Keystroke()
{
	byte incoming_char;
	 
	if (Serial.available() > 0)
	{
		incoming_char = Serial.read();
		switch (incoming_char)
		{		// need to process character appropriately	
			case 'C':
			case 'c':
			// go to input menu
			Serial.write((unsigned char *)ANSI_CODE_Clear_Whole_Screen,4); // send the string
			// back to the root.
			Reset_Menu_Vars();
			Menu_State.Current_Menu = CONFIG_SPECIAL_MENU;
			Copy_Data_From_Flash(TheConfigEpromData.Values[CONFIG_PARAM_PROFILE].Num);
			break;
			case 'r':
			case 'R':
			Serial.write((unsigned char *)ANSI_CODE_Clear_Whole_Screen,4); // send the string
			break;

      default:
      break;
		}
		return(TRUE);
	}
	else
	  return(FALSE);
}
 
bool Process_ConfigMenu_Keystroke()
{
	byte incoming_char;
	bool bigstep = FALSE;
	bool updirn = FALSE; 
	byte maxrows = CONFIG_NUMBER_PARAMS;  
	 
	if (Serial.available() > 0)
	{
    incoming_char = Serial.read();
		{
			switch (incoming_char)
			{		// need to process character appropriately
			  case 'Q':
				case 'q':	
					Serial.write((unsigned char *)ANSI_CODE_Clear_Whole_Screen,4); // send the string
					// back to the root.
					Reset_Menu_Vars();
					Menu_State.Current_Menu = ROOT_MENU;
				break;
				case 'r':
				case 'R':
					Serial.write((unsigned char *)ANSI_CODE_Clear_Whole_Screen,4); // send the string
				break;

        case '>':
        case '<':
          bigstep = TRUE;
        case ',':
        case '.': 
          if ((incoming_char == '>') || (incoming_char == '.'))
            updirn = TRUE;
            
          Edit_Field_Value (&(Config_Menu.string_selections[Menu_State.Edit_Row].Value_Stuff),
            bigstep,
            updirn,
            pgm_read_word_near(&(Config_Menu.string_selections[Menu_State.Edit_Row].Value_Stuff.Max_Val)),
            pgm_read_word_near(&(Config_Menu.string_selections[Menu_State.Edit_Row].Value_Stuff.Min_Val)) );   
          break;

        case 'E': // move down
			  case 'e':
        	Menu_State.Edit_Row = CONFIG_MENU_EDIT_START_LINE;
  				Menu_State.Editing = TRUE;
          break;

      	case 'N': // move down
			  case 'n':
			     Menu_State.Edit_Row = (Menu_State.Edit_Row < CONFIG_MENU_EDIT_END_LINE)? ++Menu_State.Edit_Row : CONFIG_MENU_EDIT_END_LINE;
        break;
        case 'P': // move up
        case 'p':
          Menu_State.Edit_Row = (Menu_State.Edit_Row > CONFIG_MENU_EDIT_START_LINE )? --Menu_State.Edit_Row : CONFIG_MENU_EDIT_START_LINE;
        break;
        
				case 'S':
				case 's':  // save and exit edit mode
  				Menu_State.Edit_Row = CONFIG_MENU_EDIT_START_LINE;
  				Menu_State.Editing = FALSE;
  				// save the store and exit edit mode
  				Copy_Data_To_Flash(TheConfigEpromData.Values[CONFIG_PARAM_PROFILE].Num);
				break;  
        default:
        break;
			}
		}
		return(TRUE);
	}
	else
	  return (FALSE);
}
 
 
bool Process_Keystroke (void)
{
	bool keystuck = FALSE;
	// just performs steering to the appropriate menu type
	switch (Menu_State.Current_Menu)
	{
		case ROOT_MENU:
		keystuck = Process_RootMenu_Keystroke();
		break;
		 
		case CONFIG_SPECIAL_MENU:
		keystuck = Process_ConfigMenu_Keystroke();
		break;		 
	}
	return(keystuck);
}
 
 
void Copy_Data_From_Flash(byte slot)
{
	byte * ptr;
  size_t i;
  size_t offset;

  offset =  0 + (uint16_t)slot * (uint16_t)EEPROM_PROFILE_SPACING;
	// needs to load from the stored array - but for now we put in the defaults
  {
    
  	ptr = (byte *)&TheDisplayEpromData;
   
  	for (i=0; i< sizeof(TheDisplayEpromData); i++)
  	*ptr++ = EEPROM.read(i+ offset);
  	 	
  	ptr = (byte *) &TheConfigEpromData;
  	for (i=(sizeof (TheDisplayEpromData));
  				i< (sizeof (TheDisplayEpromData)+ sizeof(TheConfigEpromData)); i++)
  	*ptr++ = EEPROM.read(i+ offset);
  }
 
//  Serial.print("EE Retreieved: ");
//  Serial.print((sizeof (TheDisplayEpromData)+ sizeof(TheConfigEpromData)),DEC);
//  Serial.print(" Start @: ");
//  Serial.println(offset,DEC);
};
 
void Copy_Data_To_Flash(byte slot)
{
	byte * ptr;
	size_t i;
  size_t offset;
  byte val;
  
  offset =  0 + (uint16_t) slot * (uint16_t) EEPROM_PROFILE_SPACING;
  
	ptr = (byte *) &TheDisplayEpromData;
  
	for (i=0; i< (sizeof(TheDisplayEpromData)); i++)
	  EEPROM.write(i + offset,*ptr++);
	 	
	ptr = (byte *) &TheConfigEpromData;
	for (i=( sizeof (TheDisplayEpromData));
				i< ( sizeof (TheDisplayEpromData)+ sizeof(TheConfigEpromData)); i++)
	  EEPROM.write(i+ offset,*ptr++);	

  // write the byte indcating the memory has been initialised
  Write_Var_To_Flash(STORED_SETUP_FLAG_VALUE, STORED_SETUP_FLAG_ADDRESS) ;
//  Serial.print("EE Stored: ");
//  Serial.print((sizeof (TheDisplayEpromData)+ sizeof(TheConfigEpromData)),DEC);
//  Serial.print(" Start @: ");
//  Serial.println(offset,DEC);

}

// ----------------------- GENERAL VERS  ---------------
// saving code space

void Write_Var_To_Flash(uint32_t value, uint16_t ADDRESS)
{
  uint8_t *ptr, *p;
  
  ptr = (uint8_t *) &value;
  for(p = ADDRESS; p < ADDRESS + sizeof(value); p++)
  {
    EEPROM.write(p,*ptr++);
  }
} 

uint32_t Read_Var_From_Flash (uint16_t ADDRESS)
{
  uint8_t *ptr, *p;
  uint32_t value;

  ptr = (uint8_t *) &value;
  for (p = ADDRESS; p < (ADDRESS +  sizeof(value)); p++)
    *ptr++ = (uint8_t)EEPROM.read(p);

  // Serial.print("ODO : ");
  // Serial.println(value,DEC);
  return(value);
}


void Write_Byte_To_Flash(uint8_t value, uint16_t ADDRESS)
{
    EEPROM.write(ADDRESS,value);
} 


uint8_t Read_Byte_From_Flash (uint16_t ADDRESS)
{
  uint8_t value;
  value = (uint8_t)EEPROM.read(ADDRESS);

  // Serial.print("ODO : ");
  // Serial.println(value,DEC);
  return(value);
}

// --------------------------------------------------





 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
