// menu definitions header
#include <avr/pgmspace.h>
#include "global_data.h" 
// need to firstly define the structures that are going to be
// used for the data .

// shared menu
#define MENU_FIELD_WIDTH 8    // used for spacing
#define DISPLAY_SEL_WIDTH 5
#define CONFIG_STRING_WIDTH  30
//#define MENU_STR_LEN MENU_FIELD_WIDTH + 1
#define MENU_STR_LEN 7 + 1    // shortening strings
#define MENU_SPACNG MENU_STR_LEN + 1  // field spacing
 
#define MENU_EDIT_COLOUR  COLOUR_CODE_RED
 
 
typedef unsigned char MENU_STR [MENU_STR_LEN];
  
// the items that are able to be set
#define DISPLAY_NUMBER_OF_DISPLAYS  10  // one less thna real ?
 
 
typedef enum
{
	Item,Display,Value,BOOL,
}ENTRY_TYPE;
 
typedef void (*FUNCPTR)(bool,bool);   // ptr to function taking row and column indices for edit
 
typedef struct
{
	ENTRY_TYPE   var_type;
	VALUE_UNION_TYPE *Valptr;
	unsigned int Step_Val;  // how much to step - granularity
	unsigned int Min_Val;
	unsigned int Max_Val;
	byte Display_Scale;
}MENU_FIELD_ENTRY;
 
/* -------------------- per menu ---------------------- */
 
typedef struct
{
	MENU_FIELD_ENTRY Fields[DISPLAY_NUMBER_MENU_FIELDS];
}DISPLAY_MENU_ROW;
 
 
typedef struct
{
	byte  MaxRows;
	byte  MaxCols;
	DISPLAY_MENU_ROW Menu_Rows[DISPLAY_NUMBER_MENU_ITEMS];
	FUNCPTR Execute[DISPLAY_NUMBER_MENU_FIELDS];
}DISPLAY_MENU_TOP;
 
// ------------------------ the data ----------------------------------------------

 
/* ---------------------------------------------------------- */
 
// ----------------------------- ANSI terminal codes --------
const unsigned char ANSI_CODE_Colour_Black[]  = {"[30m"};
const unsigned char ANSI_CODE_Colour_Red[]   = {"[31m"};
const unsigned char ANSI_CODE_Colour_Green[]  = {"[32m"};
//const unsigned char ANSI_CODE_Colour_Yellow[]   = {"[33m"};
const unsigned char ANSI_CODE_Colour_Blue[]  = {"[34m"};
const unsigned char ANSI_CODE_Colour_Magenta[]   = {"[35m"};
//const unsigned char ANSI_CODE_Colour_Cyan[]  = {"[36m"};
//const unsigned char ANSI_CODE_Colour_White[]   = {"[37m"};
//const unsigned char ANSI_CODE_Colour_Reset[]  = {"[0m"};
//const unsigned char ANSI_CODE_Set_Bold[]  = {"[1m"};
 
 
 
//const unsigned char ANSI_CODE_Move_Start_Line[]   = {"[0G"};
//const unsigned char ANSI_CODE_Clear_Start_Line[]  = {"[1K"};
//const unsigned char ANSI_CODE_Clear_Whole_Line[]  = {"[2K"};
//const unsigned char ANSI_CODE_Clear_Whole_Line[]  = {"[K"};
//const unsigned char ANSI_CODE_Clear_End_Line[]  = {"[0K"};
const unsigned char ANSI_CODE_Clear_Start_Screen[]  = {"[1J"};
const unsigned char ANSI_CODE_Clear_Whole_Screen[]  = {"[2J"};
//const unsigned char ANSI_CODE_Clear_End_Screen[]  = {"[0J"};
const unsigned char ANSI_CODE_Move_To_Position_begin[]  = {"["};
const unsigned char ANSI_CODE_Move_To_Position_end[]  = {"f"};
 
//const unsigned char ANSI_CODE_Set_80x25_Colour[] = {"[=3h"};
//const unsigned char ANSI_CODE_Set_40x25_Colour[] = {"[=0h"};
 
#define COLOUR_CODE_BLACK   0
#define COLOUR_CODE_RED     1
#define COLOUR_CODE_GREEN   2
#define COLOUR_CODE_YELLOW  3
#define COLOUR_CODE_BLUE    4
#define COLOUR_CODE_MAGENTA 5
#define COLOUR_CODE_CYAN    6
#define COLOUR_CODE_WHITE   7
#define COLOUR_CODE_RESET   8
 

 
// stuff for the managment of the menu state
typedef enum
{
	ROOT_MENU,
	CONFIG_SPECIAL_MENU
}MENU_TYPE;
 
typedef struct
{
	MENU_TYPE Current_Menu;
	byte Text_Index_Value;
	byte Row_Index_Value;
	byte Displayed;
	byte Edit_Row;
	byte Editing;
}MENU_STATE_VARS;
 
 
#define ROOT_MENU_LINES 3
#define CONFIG_MENU_LINES 37
#define CONFIG_MENU_EDIT_START_LINE 3 //   
#define CONFIG_MENU_EDIT_END_LINE 36 //  
 
 
typedef struct
{
	byte posx;
	byte posy;
  byte colour;
	unsigned char text[39];
} LOCATED_STRING_TYPE; 
 
// ------------------------------ ROOT MENU---------------- 
typedef struct
{ 
const LOCATED_STRING_TYPE string_selections[ROOT_MENU_LINES]; 
} ROOT_TEXT_MENU_TYPE; 
 

// ------------------------------ CONFIG MENU --------------

typedef struct 
{
	LOCATED_STRING_TYPE Text_Stuff;
	MENU_FIELD_ENTRY    Value_Stuff;
} CONFIG_TEXT_AND_EDIT_TYPE;

typedef struct
{ 
	const CONFIG_TEXT_AND_EDIT_TYPE string_selections[CONFIG_MENU_LINES]; 
} CONFIG_TEXT_MENU_TYPE; 


// ------------------------------ INPUT MENU --------------------

 
 PROGMEM const ROOT_TEXT_MENU_TYPE Root_Menu = {{
{10,2,COLOUR_CODE_BLACK,    {"Rr Refresh Display           "}},
{10,4,COLOUR_CODE_BLACK,    {"Cc Configuration Menu        "}},
}};
 


PROGMEM const CONFIG_TEXT_MENU_TYPE Config_Menu = {{
{{10,2,COLOUR_CODE_GREEN,   {"Configuration Set Menu        "}},{Value,NULL,0,0,0,1}},
{{10,3,COLOUR_CODE_BLUE,    {"eEdit sSave qQuitr rRefrsh    "}},{Value,NULL,0,0,0,1}},
{{10,4,COLOUR_CODE_BLUE,    {"nNext pPrev ,< (dn) .>(Up)    "}},{Value,NULL,0,0,0,1}},

// inputs
{{10,6,COLOUR_CODE_BLUE,     {"4 BUTTON CONTROL MODE        "}},{Value,&TheConfigEpromData.Values[CONFIG_PARAM_4_BUTTON_MODE],1,0,4,1}},

{{10,8,COLOUR_CODE_BLACK,   {"HIGH Beam Input POL Hi        "}},{BOOL,&TheConfigEpromData.Values[CONFIG_PARAM_HIGH_POL],1,0,1,1}},
{{10,9,COLOUR_CODE_BLACK,   {"HIGH Beam Input STATE         "}},{BOOL,&TheConfigEpromData.Values[CONFIG_PARAM_HIGH_MODE],1,0,1,1}},

{{10,10,COLOUR_CODE_BLUE,    {"STARTER Input POL Hi          "}},{BOOL,&TheConfigEpromData.Values[CONFIG_PARAM_STARTER_POL],1,0,1,1}},
{{10,11,COLOUR_CODE_BLUE,    {"STARTER Input STATE           "}},{BOOL,&TheConfigEpromData.Values[CONFIG_PARAM_STARTER_MODE],1,0,1,1}},

{{10,12,COLOUR_CODE_BLACK,  {"LEFT Ind Input POL Hi        "}},{BOOL,&TheConfigEpromData.Values[CONFIG_PARAM_LEFT_POL],1,0,1,1}},
{{10,13,COLOUR_CODE_BLACK,  {"LEFT Ind Input STATE         "}},{BOOL,&TheConfigEpromData.Values[CONFIG_PARAM_LEFT_MODE],1,0,1,1}},

{{10,14,COLOUR_CODE_BLACK,  {"RIGHT Ind Input POL Hi       "}},{BOOL,&TheConfigEpromData.Values[CONFIG_PARAM_RIGHT_POL],1,0,1,1}},
{{10,15,COLOUR_CODE_BLACK,  {"Right Ind Input STATE        "}},{BOOL,&TheConfigEpromData.Values[CONFIG_PARAM_RIGHT_MODE],1,0,1,1}},

{{10,16,COLOUR_CODE_BLUE,   {"HORN Input POL Hi            "}},{BOOL,&TheConfigEpromData.Values[CONFIG_PARAM_HORN_POL],1,0,1,1}},
{{10,17,COLOUR_CODE_BLUE,   {"HORN Input STATE             "}},{BOOL,&TheConfigEpromData.Values[CONFIG_PARAM_HORN_MODE],1,0,1,1}},

{{10,18,COLOUR_CODE_BLACK,  {"KILL Input POL Hi            "}},{BOOL,&TheConfigEpromData.Values[CONFIG_PARAM_KILL_POL],1,0,1,1}},
{{10,19,COLOUR_CODE_BLACK,  {"KILL Input STATE             "}},{BOOL,&TheConfigEpromData.Values[CONFIG_PARAM_KILL_MODE],1,0,1,1}},

{{10,20,COLOUR_CODE_BLUE,   {"BRAKE Input POL Hi           "}},{BOOL,&TheConfigEpromData.Values[CONFIG_PARAM_BRAKE_POL],1,0,1,1}},
{{10,21,COLOUR_CODE_BLUE,   {"BRAKE Input STATE            "}},{BOOL,&TheConfigEpromData.Values[CONFIG_PARAM_BRAKE_MODE],1,0,1,1}},

{{10,22,COLOUR_CODE_BLACK,  {"LIGHTS Input POL Hi          "}},{BOOL,&TheConfigEpromData.Values[CONFIG_PARAM_LIGHTS_POL],1,0,1,1}},
{{10,23,COLOUR_CODE_BLACK,  {"LIGHTS Input STATE           "}},{BOOL,&TheConfigEpromData.Values[CONFIG_PARAM_LIGHTS_MODE],1,0,1,1}},

{{10,24,COLOUR_CODE_BLUE,   {"GATE Input POL Hi            "}},{BOOL,&TheConfigEpromData.Values[CONFIG_PARAM_GATE_POL],1,0,1,1}},
{{10,25,COLOUR_CODE_BLUE,   {"GATE Input STATE             "}},{BOOL,&TheConfigEpromData.Values[CONFIG_PARAM_GATE_MODE],1,1,1,1}},

// outputs
{{10,28,COLOUR_CODE_BLACK,  {"LIGHTS Off When Crank        "}},{BOOL,&TheConfigEpromData.Values[CONFIG_PARAM_LIGHTS_OFF_CRANK],1,0,1,1}},
{{10,29,COLOUR_CODE_BLACK,  {"LOW Beam Active with High    "}},{BOOL,&TheConfigEpromData.Values[CONFIG_PARAM_LOW_OUT_ACT],1,0,1,1}},
{{10,30,COLOUR_CODE_BLACK,  {"LIGHTS On with Power On      "}},{BOOL,&TheConfigEpromData.Values[CONFIG_PARAM_LIGHT_ON_POW],1,0,1,1}},
{{10,31,COLOUR_CODE_BLACK,  {"MODULATE Low Beam            "}},{BOOL,&TheConfigEpromData.Values[CONFIG_PARAM_MOD_LOW_BEAM],1,0,1,1}},
{{10,32,COLOUR_CODE_BLACK,  {"MODULATION Depth % (30-100)  "}},{Value,&TheConfigEpromData.Values[CONFIG_PARAM_MOD_LOW_DEPTH],10,30,100,1}},

{{10,33,COLOUR_CODE_BLUE,   {"STARTER GATED                "}},{BOOL,&TheConfigEpromData.Values[CONFIG_PARAM_STARTER_GATE],1,0,1,1}},
{{10,34,COLOUR_CODE_BLUE,   {"STARTER Inhibited by KILL    "}},{BOOL,&TheConfigEpromData.Values[CONFIG_PARAM_STARTER_INH_KILL],1,0,1,1}},

{{10,36,COLOUR_CODE_BLACK,  {"INDICATOR MAX FLASH limit    "}},{BOOL,&TheConfigEpromData.Values[CONFIG_PARAM_IND_MAX_FLASH_FLAG],1,0,1,1}},
{{10,37,COLOUR_CODE_BLACK,  {"IND MAX FLASH Time value (S) "}},{Value,&TheConfigEpromData.Values[CONFIG_PARAM_IND_MAX_LIM_VALUE],1,10,90,1}},
{{10,38,COLOUR_CODE_BLACK,  {"INDICATOR FLASH RATE Hz (1-5)"}},{Value,&TheConfigEpromData.Values[CONFIG_PARAM_IND_FLASH_RATE_VALUE],1,0,5,1}},

{{10,39,COLOUR_CODE_BLUE,   {"BRAKE Warning FLASH          "}},{BOOL,&TheConfigEpromData.Values[CONFIG_PARAM_BRAKE_WARN_FLASH_FLAG],1,0,1,1}},
{{10,40,COLOUR_CODE_BLUE,   {"BRAKE WARN FLASH number      "}},{Value,&TheConfigEpromData.Values[CONFIG_PARAM_BRAKE_WARN_FLASH_VALUE],1,0,10,1}},
{{10,41,COLOUR_CODE_BLUE,   {"BRAKE Off When IND On        "}},{BOOL,&TheConfigEpromData.Values[CONFIG_PARAM_BRAKE_WHEN_FLASH_FLAG],1,0,1,1}},

{{10,42,COLOUR_CODE_BLACK,   {"Enable HEATED GRIP function   "}},{BOOL,&TheConfigEpromData.Values[CONFIG_PARAM_HEATED_GRIP_ON],1,0,1,1}},
{{10,43,COLOUR_CODE_BLACK,   {"Heated Grip heat Steps (2-5)  "}},{Value,&TheConfigEpromData.Values[CONFIG_PARAM_HEATED_GRIP_STEPS],1,2,5,1}},

}};



 
